@@ -1,16 +1,13 @@
-import logging
 from time import sleep
 
 from imapclient.exceptions import IMAPClientError
 from mailsuite.imap import IMAPClient
 from socket import timeout
 
+from parsedmarc.log import logger
 from parsedmarc.mail.mailbox_connection import MailboxConnection
 
 
-logger = logging.getLogger("parsedmarc")
-
-
 class IMAPConnection(MailboxConnection):
     def __init__(self,
                  host=None,
@@ -32,7 +29,7 @@ class IMAPConnection(MailboxConnection):
     def create_folder(self, folder_name: str):
         self._client.create_folder(folder_name)
 
-    def fetch_messages(self, reports_folder: str):
+    def fetch_messages(self, reports_folder: str, **kwargs):
         self._client.select_folder(reports_folder)
         return self._client.search()
 
